@@ -18,6 +18,9 @@ import json
 import zarr
 import requests
 import dask.array as da
+import warnings
+
+from dask.diagnostics import ProgressBar
 from vispy.color import Colormap
 
 from urllib.parse import urlparse
@@ -49,16 +52,18 @@ def napari_get_reader(path: PathLike) -> Optional[ReaderFunction]:
     """
     if isinstance(path, list):
         path = path[0]
+    instance = parse_url(path)
+    if instance.is_zarr():
+        return instance.get_reader_function()
 
+
+def parse_url(path):
     result = urlparse(path)
     if result.scheme in ("", "file"):
         # Strips 'file://' if necessary
-        instance = LocalZarr(result.path)
+        return LocalZarr(result.path)
     else:
-        instance = RemoteZarr(path)
-
-    if instance.is_zarr():
-        return instance.get_reader_function()
+        return RemoteZarr(path)
 
 
 class BaseZarr:
@@ -68,8 +73,23 @@ class BaseZarr:
         self.zarray = self.get_json(".zarray")
         self.zgroup = self.get_json(".zgroup")
         if self.zgroup:
-            self.image_data = self.get_json("omero.json")
             self.root_attrs = self.get_json(".zattrs")
+            if "omero" in self.root_attrs:
+                self.image_data = self.root_attrs["omero"]
+                # TODO: start checking metadata version
+            else:
+                # Backup location that can be removed in the future.
+                warnings.warn("deprecated loading of omero.josn",
+                              DeprecationWarning)
+                self.image_data = self.get_json("omero.json")
+
+    def __str__(self):
+        suffix = ""
+        if self.zgroup:
+            suffix += " [zgroup]"
+        if self.zarray:
+            suffix += " [zarray]"
+        return f"{self.zarr_path}{suffix}"
 
     def is_zarr(self):
         return self.zarray or self.zgroup
@@ -82,7 +102,7 @@ class BaseZarr:
 
     def get_reader_function(self):
         if not self.is_zarr():
-            raise Exception("not a zarr")
+            raise Exception(f"not a zarr: {self}")
         return self.reader_function
 
     def reader_function(self, path: PathLike) -> List[LayerData]:
@@ -163,9 +183,58 @@ class RemoteZarr(BaseZarr):
     def get_json(self, subpath):
         rsp = requests.get(f"{self.zarr_path}{subpath}")
         try:
-            if rsp.status_code == 403:  # file doesn't exist
+            if rsp.status_code in (403, 404):  # file doesn't exist
                 return {}
             return rsp.json()
         except:
-            print("FIXME", rsp.text, dir(rsp))
+            print("FIXME", rsp.status_code, rsp.text)
             return {}
+
+
+def info(path):
+    """
+    print information about the ome-zarr fileset
+    """
+    zarr = parse_url(path)
+    if not zarr.is_ome_zarr():
+        print(f"not an ome-zarr: {zarr}")
+        return
+    reader = zarr.get_reader_function()
+    data = reader(path)
+    print(data)
+
+
+def download(path, output_dir='.', zarr_name=''):
+    """
+    download zarr from URL
+    """
+    omezarr = parse_url(path)
+    if not omezarr.is_ome_zarr():
+        print(f"not an ome-zarr: {path}")
+
+    image_id = omezarr.image_data.get('id', 'unknown')
+    print('image_id', image_id)
+    if not zarr_name:
+        zarr_name = f'{image_id}.zarr'
+
+    try:
+        datasets = [x['path'] for x in omezarr.root_attrs["multiscales"][0]["datasets"]]
+    except KeyError:
+        datasets = ["0"]
+    print('datasets', datasets)
+    resolutions = [da.from_zarr(path, component=str(i)) for i in datasets]
+    # levels = list(range(len(resolutions)))
+
+    target_dir = os.path.join(output_dir, f'{zarr_name}')
+    print(f'downloading to {target_dir}')
+
+    pbar = ProgressBar()
+    for dataset, data in reversed(list(zip(datasets, resolutions))):
+        print(f'resolution {dataset}...')
+        with pbar:
+            data.to_zarr(os.path.join(target_dir, dataset))
+
+    with open(os.path.join(target_dir, '.zgroup'), 'w') as f:
+        f.write(json.dumps(omezarr.zgroup))
+    with open(os.path.join(target_dir, '.zattrs'), 'w') as f:
+        f.write(json.dumps(omezarr.root_attrs))
